@@ -1,22 +1,25 @@
 // ctl_value.rs
 
 #[cfg(target_os = "freebsd")]
-use temperature::Temperature;
+use crate::temperature::Temperature;
+use enum_as_inner::EnumAsInner;
 
 /// An Enum that holds all values returned by sysctl calls.
-/// Extract inner value with `if let` or `match`.
+/// Extract inner value with accessors like `as_int()`.
 ///
 /// # Example
 ///
 /// ```
 /// # use sysctl::Sysctl;
 /// if let Ok(ctl) = sysctl::Ctl::new("kern.osrevision") {
-///     if let Ok(sysctl::CtlValue::Int(val)) = ctl.value() {
-///         println!("Value: {}", val);
+///     if let Ok(r) = ctl.value() {
+///         if let Some(val) = r.as_int() {
+///             println!("Value: {}", val);
+///         }
 ///     }
 /// }
 /// ```
-#[derive(Debug, PartialEq, PartialOrd)]
+#[derive(Debug, EnumAsInner, PartialEq, PartialOrd)]
 pub enum CtlValue {
     None,
     Node(Vec<u8>),
@@ -71,6 +74,8 @@ mod tests_linux {
 
     #[test]
     fn ctl_value_string() {
+        // NOTE: Some linux distributions require Root permissions
+        //        e.g Debian.
         let output = std::process::Command::new("sysctl")
             .arg("-n")
             .arg("kernel.version")
@@ -92,7 +97,7 @@ mod tests_linux {
     }
 }
 
-#[cfg(all(test, any(target_os = "freebsd", target_os = "macos")))]
+#[cfg(all(test, any(target_os = "freebsd", target_os = "macos", target_os = "ios", target_os = "tvos", target_os = "visionos")))]
 mod tests_unix {
     use crate::sys;
     use crate::Sysctl;
