@@ -1,8 +1,8 @@
 // linux/ctl_iter.rs
 
 use super::ctl::Ctl;
-use ctl_error::SysctlError;
-use traits::Sysctl;
+use crate::ctl_error::SysctlError;
+use crate::traits::Sysctl;
 
 /// An iterator over Sysctl entries.
 pub struct CtlIter {
@@ -108,6 +108,8 @@ mod tests {
 
     #[test]
     fn ctl_iter_below_compare_outputs() {
+        // NOTE: Some linux distributions require Root permissions
+        //        e.g Debian.
         let output = std::process::Command::new("sysctl")
             .arg("user")
             .output()
