@@ -1,33 +1,32 @@
 // unix/funcs.rs
 
 use byteorder::{ByteOrder, WriteBytesExt};
-use consts::*;
-use ctl_error::*;
-use ctl_info::*;
-use ctl_type::*;
-use ctl_value::*;
+use crate::consts::*;
+use crate::ctl_error::*;
+use crate::ctl_info::*;
+use crate::ctl_type::*;
+use crate::ctl_value::*;
+#[cfg(any(target_os = "macos", target_os = "ios", target_os = "tvos", target_os = "visionos"))]
+use std::ffi::CString;
 
 #[cfg(target_os = "freebsd")]
-use temperature::*;
+use crate::temperature::*;
 
-#[cfg(not(target_os = "macos"))]
 pub fn name2oid(name: &str) -> Result<Vec<libc::c_int>, SysctlError> {
-    // Request command for OID
-    let oid: [libc::c_int; 2] = [0, 3];
-
-    let mut len: usize = CTL_MAXNAME as usize * std::mem::size_of::<libc::c_int>();
-
     // We get results in this vector
-    let mut res: Vec<libc::c_int> = vec![0; CTL_MAXNAME as usize];
+    let mut len: usize = CTL_MAXNAME as usize;
+    let mut res = Vec::<libc::c_int>::with_capacity(len);
+    let rcname = std::ffi::CString::new(name);
+    if rcname.is_err() {
+        return Err(SysctlError::NotFound(name.to_owned()));
+    }
+    let cname = rcname.unwrap();
 
     let ret = unsafe {
-        libc::sysctl(
-            oid.as_ptr(),
-            2,
-            res.as_mut_ptr() as *mut libc::c_void,
-            &mut len,
-            name.as_ptr() as *const libc::c_void,
-            name.len(),
+        libc::sysctlnametomib(
+            cname.as_ptr() as *const libc::c_char,
+            res.as_mut_ptr(),
+            &mut len
         )
     };
     if ret < 0 {
@@ -36,55 +35,14 @@ pub fn name2oid(name: &str) -> Result<Vec<libc::c_int>, SysctlError> {
             std::io::ErrorKind::NotFound => SysctlError::NotFound(name.into()),
             _ => SysctlError::IoError(e),
         });
+    } else {
+        unsafe { res.set_len(len); }
     }
 
-    // len is in bytes, convert to number of libc::c_ints
-    len /= std::mem::size_of::<libc::c_int>();
-
-    // Trim result vector
-    res.truncate(len);
-
     Ok(res)
 }
 
-#[cfg(target_os = "macos")]
-pub fn name2oid(name: &str) -> Result<Vec<libc::c_int>, SysctlError> {
-    // Request command for OID
-    let mut oid: [libc::c_int; 2] = [0, 3];
-
-    let mut len: usize = CTL_MAXNAME as usize * std::mem::size_of::<libc::c_int>();
-
-    // We get results in this vector
-    let mut res: Vec<libc::c_int> = vec![0; CTL_MAXNAME as usize];
-
-    let ret = unsafe {
-        libc::sysctl(
-            oid.as_mut_ptr(),
-            2,
-            res.as_mut_ptr() as *mut libc::c_void,
-            &mut len,
-            name.as_ptr() as *mut libc::c_void,
-            name.len(),
-        )
-    };
-    if ret < 0 {
-        let e = std::io::Error::last_os_error();
-        return Err(match e.kind() {
-            std::io::ErrorKind::NotFound => SysctlError::NotFound(name.into()),
-            _ => SysctlError::IoError(e),
-        });
-    }
-
-    // len is in bytes, convert to number of libc::c_ints
-    len /= std::mem::size_of::<libc::c_int>();
-
-    // Trim result vector
-    res.truncate(len);
-
-    Ok(res)
-}
-
-#[cfg(not(target_os = "macos"))]
+#[cfg(not(any(target_os = "macos", target_os = "ios", target_os = "tvos", target_os = "visionos")))]
 pub fn oidfmt(oid: &[libc::c_int]) -> Result<CtlInfo, SysctlError> {
     // Request command for type info
     let mut qoid: Vec<libc::c_int> = vec![0, 4];
@@ -120,7 +78,7 @@ pub fn oidfmt(oid: &[libc::c_int]) -> Result<CtlInfo, SysctlError> {
             Err(e) => return Err(SysctlError::InvalidCStr(e)),
         };
 
-    #[cfg_attr(feature = "cargo-clippy", allow(clippy::redundant_field_names))]
+    #[allow(clippy::redundant_field_names)]
     let s = CtlInfo {
         ctl_type: CtlType::from(ctltype_val),
         fmt: fmt,
@@ -129,7 +87,7 @@ pub fn oidfmt(oid: &[libc::c_int]) -> Result<CtlInfo, SysctlError> {
     Ok(s)
 }
 
-#[cfg(target_os = "macos")]
+#[cfg(any(target_os = "macos", target_os = "ios", target_os = "tvos", target_os = "visionos"))]
 pub fn oidfmt(oid: &[libc::c_int]) -> Result<CtlInfo, SysctlError> {
     // Request command for type info
     let mut qoid: Vec<libc::c_int> = vec![0, 4];
@@ -159,7 +117,15 @@ pub fn oidfmt(oid: &[libc::c_int]) -> Result<CtlInfo, SysctlError> {
     let ctltype_val = kind & CTLTYPE as u32;
 
     // 'fmt' is after 'Kind' in result buffer
-    let fmt: String = match std::str::from_utf8(&buf[std::mem::size_of::<u32>()..buf_len]) {
+    let bytes = &buf[std::mem::size_of::<u32>()..buf_len];
+
+    // Ensure we drop the '\0' byte if there are any.
+    let len = match bytes.iter().position(|c| *c == 0) {
+        Some(index) => index,
+        _ => bytes.len(),
+    };
+
+    let fmt: String = match std::str::from_utf8(&bytes[..len]) {
         Ok(x) => x.to_owned(),
         Err(e) => return Err(SysctlError::Utf8Error(e)),
     };
@@ -172,7 +138,7 @@ pub fn oidfmt(oid: &[libc::c_int]) -> Result<CtlInfo, SysctlError> {
     Ok(s)
 }
 
-#[cfg(not(target_os = "macos"))]
+#[cfg(not(any(target_os = "macos", target_os = "ios", target_os = "tvos", target_os = "visionos")))]
 pub fn value_oid(oid: &[i32]) -> Result<CtlValue, SysctlError> {
     let info: CtlInfo = oidfmt(&oid)?;
 
@@ -261,7 +227,7 @@ pub fn value_oid(oid: &[i32]) -> Result<CtlValue, SysctlError> {
     }
 }
 
-#[cfg(target_os = "macos")]
+#[cfg(any(target_os = "macos", target_os = "ios", target_os = "tvos", target_os = "visionos"))]
 pub fn value_oid(oid: &mut Vec<i32>) -> Result<CtlValue, SysctlError> {
     let info: CtlInfo = oidfmt(&oid)?;
 
@@ -310,22 +276,29 @@ pub fn value_oid(oid: &mut Vec<i32>) -> Result<CtlValue, SysctlError> {
 
     // Confirm that we did not read out of bounds
     assert!(new_val_len <= val_len);
-    // Confirm that we got the bytes we requested
+    // The call can sometimes return bytes that are smaller than initially indicated, so it should
+    // be safe to truncate it.  See a similar approach in golang module:
+    // https://github.com/golang/sys/blob/43e60d72a8e2bd92ee98319ba9a384a0e9837c08/unix/syscall_bsd.go#L545-L548
     if new_val_len < val_len {
-        return Err(SysctlError::ShortRead {
-            read: new_val_len,
-            reported: val_len,
-        });
+        val.truncate(new_val_len);
     }
 
     // Wrap in Enum and return
     match info.ctl_type {
         CtlType::None => Ok(CtlValue::None),
         CtlType::Node => Ok(CtlValue::Node(val)),
-        CtlType::Int => Ok(CtlValue::Int(byteorder::LittleEndian::read_i32(&val))),
-        CtlType::String => match std::str::from_utf8(&val[..val.len() - 1]) {
-            Ok(s) => Ok(CtlValue::String(s.into())),
-            Err(e) => Err(SysctlError::Utf8Error(e)),
+        CtlType::Int => match info.fmt.as_str() {
+            "I" => Ok(CtlValue::Int(byteorder::LittleEndian::read_i32(&val))),
+            "IU" => Ok(CtlValue::Uint(byteorder::LittleEndian::read_u32(&val))),
+            "L" => Ok(CtlValue::Long(byteorder::LittleEndian::read_i64(&val))),
+            "LU" => Ok(CtlValue::Ulong(byteorder::LittleEndian::read_u64(&val))),
+            _ => Ok(CtlValue::None),
+        }
+        CtlType::String => match val.len() {
+            0 => Ok(CtlValue::String("".to_string())),
+            l => std::str::from_utf8(&val[..l - 1])
+                .map_err(SysctlError::Utf8Error)
+                .map(|s| CtlValue::String(s.into())),
         },
         CtlType::S64 => Ok(CtlValue::S64(byteorder::LittleEndian::read_i64(&val))),
         CtlType::Struct => Ok(CtlValue::Struct(val)),
@@ -339,12 +312,10 @@ pub fn value_oid(oid: &mut Vec<i32>) -> Result<CtlValue, SysctlError> {
         CtlType::S16 => Ok(CtlValue::S16(byteorder::LittleEndian::read_i16(&val))),
         CtlType::S32 => Ok(CtlValue::S32(byteorder::LittleEndian::read_i32(&val))),
         CtlType::U32 => Ok(CtlValue::U32(byteorder::LittleEndian::read_u32(&val))),
-        #[cfg(not(target_os = "macos"))]
-        _ => Err(SysctlError::UnknownType),
     }
 }
 
-#[cfg(not(target_os = "macos"))]
+#[cfg(not(any(target_os = "macos", target_os = "ios", target_os = "tvos", target_os = "visionos")))]
 pub fn value_oid_as<T>(oid: &[i32]) -> Result<Box<T>, SysctlError> {
     let val_enum = value_oid(oid)?;
 
@@ -388,7 +359,7 @@ pub fn value_oid_as<T>(oid: &[i32]) -> Result<Box<T>, SysctlError> {
     }
 }
 
-#[cfg(target_os = "macos")]
+#[cfg(any(target_os = "macos", target_os = "ios", target_os = "tvos", target_os = "visionos"))]
 pub fn value_oid_as<T>(oid: &mut Vec<i32>) -> Result<Box<T>, SysctlError> {
     let val_enum = value_oid(oid)?;
 
@@ -432,6 +403,131 @@ pub fn value_oid_as<T>(oid: &mut Vec<i32>) -> Result<Box<T>, SysctlError> {
     }
 }
 
+#[cfg(any(target_os = "macos", target_os = "ios", target_os = "tvos", target_os = "visionos"))]
+pub fn value_name(name: &str, ctl_type: CtlType, fmt: &str) -> Result<CtlValue, SysctlError> {
+    let name = CString::new(name)?;
+
+    // First get size of value in bytes
+    let mut val_len = 0;
+    let ret = unsafe {
+        libc::sysctlbyname(
+            name.as_ptr(),
+            std::ptr::null_mut(),
+            &mut val_len,
+            std::ptr::null_mut(),
+            0,
+        )
+    };
+    if ret < 0 {
+        return Err(SysctlError::IoError(std::io::Error::last_os_error()));
+    }
+
+    // If the length reported is shorter than the type we will convert it into,
+    // byteorder::LittleEndian::read_* will panic. Therefore, expand the value length to at
+    // Least the size of the value.
+    let val_minsize = std::cmp::max(val_len, ctl_type.min_type_size());
+
+    // Then get value
+    let mut val: Vec<libc::c_uchar> = vec![0; val_minsize];
+    let mut new_val_len = val_len;
+    let ret = unsafe {
+        libc::sysctlbyname(
+            name.as_ptr(),
+            val.as_mut_ptr() as *mut libc::c_void,
+            &mut new_val_len,
+            std::ptr::null_mut(),
+            0,
+        )
+    };
+    if ret < 0 {
+        return Err(SysctlError::IoError(std::io::Error::last_os_error()));
+    }
+
+    // Confirm that we did not read out of bounds
+    assert!(new_val_len <= val_len);
+    // The call can sometimes return bytes that are smaller than initially indicated, so it should
+    // be safe to truncate it.  See a similar approach in golang module:
+    // https://github.com/golang/sys/blob/43e60d72a8e2bd92ee98319ba9a384a0e9837c08/unix/syscall_bsd.go#L545-L548
+    if new_val_len < val_len {
+        val.truncate(new_val_len);
+    }
+
+    // Wrap in Enum and return
+    match ctl_type {
+        CtlType::None => Ok(CtlValue::None),
+        CtlType::Node => Ok(CtlValue::Node(val)),
+        CtlType::Int => match fmt {
+            "I" => Ok(CtlValue::Int(byteorder::LittleEndian::read_i32(&val))),
+            "IU" => Ok(CtlValue::Uint(byteorder::LittleEndian::read_u32(&val))),
+            "L" => Ok(CtlValue::Long(byteorder::LittleEndian::read_i64(&val))),
+            "LU" => Ok(CtlValue::Ulong(byteorder::LittleEndian::read_u64(&val))),
+            _ => Ok(CtlValue::None),
+        }
+        CtlType::String => match val.len() {
+            0 => Ok(CtlValue::String("".to_string())),
+            l => std::str::from_utf8(&val[..l - 1])
+                .map_err(SysctlError::Utf8Error)
+                .map(|s| CtlValue::String(s.into())),
+        },
+        CtlType::S64 => Ok(CtlValue::S64(byteorder::LittleEndian::read_i64(&val))),
+        CtlType::Struct => Ok(CtlValue::Struct(val)),
+        CtlType::Uint => Ok(CtlValue::Uint(byteorder::LittleEndian::read_u32(&val))),
+        CtlType::Long => Ok(CtlValue::Long(byteorder::LittleEndian::read_i64(&val))),
+        CtlType::Ulong => Ok(CtlValue::Ulong(byteorder::LittleEndian::read_u64(&val))),
+        CtlType::U64 => Ok(CtlValue::U64(byteorder::LittleEndian::read_u64(&val))),
+        CtlType::U8 => Ok(CtlValue::U8(val[0])),
+        CtlType::U16 => Ok(CtlValue::U16(byteorder::LittleEndian::read_u16(&val))),
+        CtlType::S8 => Ok(CtlValue::S8(val[0] as i8)),
+        CtlType::S16 => Ok(CtlValue::S16(byteorder::LittleEndian::read_i16(&val))),
+        CtlType::S32 => Ok(CtlValue::S32(byteorder::LittleEndian::read_i32(&val))),
+        CtlType::U32 => Ok(CtlValue::U32(byteorder::LittleEndian::read_u32(&val))),
+    }
+}
+
+#[cfg(any(target_os = "macos", target_os = "ios", target_os = "tvos", target_os = "visionos"))]
+pub fn value_name_as<T>(name: &str, ctl_type: CtlType, fmt: &str) -> Result<Box<T>, SysctlError> {
+    let val_enum = value_name(name, ctl_type, fmt)?;
+
+    // Some structs are apparently reported as Node so this check is invalid..
+    // let ctl_type = CtlType::from(&val_enum);
+    // assert_eq!(CtlType::Struct, ctl_type, "Error type is not struct/opaque");
+
+    // TODO: refactor this when we have better clue to what's going on
+    if let CtlValue::Struct(val) = val_enum {
+        // Make sure we got correct data size
+        assert_eq!(
+            std::mem::size_of::<T>(),
+            val.len(),
+            "Error memory size mismatch. Size of struct {}, size of data retrieved {}.",
+            std::mem::size_of::<T>(),
+            val.len()
+        );
+
+        // val is Vec<u8>
+        let val_array: Box<[u8]> = val.into_boxed_slice();
+        let val_raw: *mut T = Box::into_raw(val_array) as *mut T;
+        let val_box: Box<T> = unsafe { Box::from_raw(val_raw) };
+        Ok(val_box)
+    } else if let CtlValue::Node(val) = val_enum {
+        // Make sure we got correct data size
+        assert_eq!(
+            std::mem::size_of::<T>(),
+            val.len(),
+            "Error memory size mismatch. Size of struct {}, size of data retrieved {}.",
+            std::mem::size_of::<T>(),
+            val.len()
+        );
+
+        // val is Vec<u8>
+        let val_array: Box<[u8]> = val.into_boxed_slice();
+        let val_raw: *mut T = Box::into_raw(val_array) as *mut T;
+        let val_box: Box<T> = unsafe { Box::from_raw(val_raw) };
+        Ok(val_box)
+    } else {
+        Err(SysctlError::ExtractionError)
+    }
+}
+
 fn value_to_bytes(value: CtlValue) -> Result<Vec<u8>, SysctlError> {
     // TODO rest of the types
     match value {
@@ -460,7 +556,7 @@ fn value_to_bytes(value: CtlValue) -> Result<Vec<u8>, SysctlError> {
     }
 }
 
-#[cfg(not(target_os = "macos"))]
+#[cfg(not(any(target_os = "macos", target_os = "ios", target_os = "tvos", target_os = "visionos")))]
 pub fn set_oid_value(oid: &[libc::c_int], value: CtlValue) -> Result<CtlValue, SysctlError> {
     let info: CtlInfo = oidfmt(&oid)?;
 
@@ -496,7 +592,7 @@ pub fn set_oid_value(oid: &[libc::c_int], value: CtlValue) -> Result<CtlValue, S
     self::value_oid(oid)
 }
 
-#[cfg(target_os = "macos")]
+#[cfg(any(target_os = "macos", target_os = "ios", target_os = "tvos", target_os = "visionos"))]
 pub fn set_oid_value(oid: &mut Vec<libc::c_int>, value: CtlValue) -> Result<CtlValue, SysctlError> {
     let info: CtlInfo = oidfmt(&oid)?;
 
@@ -506,10 +602,23 @@ pub fn set_oid_value(oid: &mut Vec<libc::c_int>, value: CtlValue) -> Result<CtlV
     }
 
     let ctl_type = CtlType::from(&value);
+
+    // Get the correct ctl type based on the format string
+    let info_ctl_type = match info.ctl_type {
+        CtlType::Int => match info.fmt.as_str() {
+            "I" => CtlType::Int,
+            "IU" => CtlType::Uint,
+            "L" => CtlType::Long,
+            "LU" => CtlType::Ulong,
+            _ => return Err(SysctlError::MissingImplementation),
+        }
+        ctl_type => ctl_type,
+    };
+
     assert_eq!(
-        info.ctl_type, ctl_type,
+        info_ctl_type, ctl_type,
         "Error type mismatch. Type given {:?}, sysctl type: {:?}",
-        ctl_type, info.ctl_type
+        ctl_type, info_ctl_type
     );
 
     let bytes = value_to_bytes(value)?;
@@ -533,7 +642,55 @@ pub fn set_oid_value(oid: &mut Vec<libc::c_int>, value: CtlValue) -> Result<CtlV
     self::value_oid(oid)
 }
 
-#[cfg(not(target_os = "macos"))]
+#[cfg(any(target_os = "macos", target_os = "ios", target_os = "tvos", target_os = "visionos"))]
+pub fn set_name_value(
+    name: &str,
+    info_ctl_type: CtlType,
+    fmt: &str,
+    value: CtlValue,
+) -> Result<CtlValue, SysctlError> {
+    let c_name = CString::new(name)?;
+    let ctl_type = CtlType::from(&value);
+
+    // Get the correct ctl type based on the format string
+    let info_ctl_type = match info_ctl_type {
+        CtlType::Int => match fmt {
+            "I" => CtlType::Int,
+            "IU" => CtlType::Uint,
+            "L" => CtlType::Long,
+            "LU" => CtlType::Ulong,
+            _ => return Err(SysctlError::MissingImplementation),
+        }
+        ctl_type => ctl_type,
+    };
+
+    assert_eq!(
+        info_ctl_type, ctl_type,
+        "Error type mismatch. Type given {:?}, sysctl type: {:?}",
+        ctl_type, info_ctl_type
+    );
+
+    let bytes = value_to_bytes(value)?;
+
+    // Set value
+    let ret = unsafe {
+        libc::sysctlbyname(
+            c_name.as_ptr(),
+            std::ptr::null_mut(),
+            std::ptr::null_mut(),
+            bytes.as_ptr() as *mut libc::c_void,
+            bytes.len(),
+        )
+    };
+    if ret < 0 {
+        return Err(SysctlError::IoError(std::io::Error::last_os_error()));
+    }
+
+    // Get the new value and return for confirmation
+    self::value_name(name, ctl_type, fmt)
+}
+
+#[cfg(not(any(target_os = "macos", target_os = "ios", target_os = "tvos", target_os = "visionos")))]
 pub fn oid2description(oid: &[libc::c_int]) -> Result<String, SysctlError> {
     // Request command for description
     let mut qoid: Vec<libc::c_int> = vec![0, 5];
@@ -563,7 +720,7 @@ pub fn oid2description(oid: &[libc::c_int]) -> Result<String, SysctlError> {
     }
 }
 
-#[cfg(not(target_os = "macos"))]
+#[cfg(not(any(target_os = "macos", target_os = "ios", target_os = "tvos", target_os = "visionos")))]
 pub fn oid2name(oid: &[libc::c_int]) -> Result<String, SysctlError> {
     // Request command for name
     let mut qoid: Vec<libc::c_int> = vec![0, 1];
@@ -593,7 +750,7 @@ pub fn oid2name(oid: &[libc::c_int]) -> Result<String, SysctlError> {
     }
 }
 
-#[cfg(target_os = "macos")]
+#[cfg(any(target_os = "macos", target_os = "ios", target_os = "tvos", target_os = "visionos"))]
 pub fn oid2name(oid: &Vec<libc::c_int>) -> Result<String, SysctlError> {
     // Request command for name
     let mut qoid: Vec<libc::c_int> = vec![0, 1];
@@ -623,7 +780,7 @@ pub fn oid2name(oid: &Vec<libc::c_int>) -> Result<String, SysctlError> {
     }
 }
 
-#[cfg(not(target_os = "macos"))]
+#[cfg(not(any(target_os = "macos", target_os = "ios", target_os = "tvos", target_os = "visionos")))]
 pub fn next_oid(oid: &[libc::c_int]) -> Result<Option<Vec<libc::c_int>>, SysctlError> {
     // Request command for next oid
     let mut qoid: Vec<libc::c_int> = vec![0, 2];
@@ -662,7 +819,7 @@ pub fn next_oid(oid: &[libc::c_int]) -> Result<Option<Vec<libc::c_int>>, SysctlE
     Ok(Some(res))
 }
 
-#[cfg(target_os = "macos")]
+#[cfg(any(target_os = "macos", target_os = "ios", target_os = "tvos", target_os = "visionos"))]
 pub fn next_oid(oid: &Vec<libc::c_int>) -> Result<Option<Vec<libc::c_int>>, SysctlError> {
     // Request command for next oid
     let mut qoid: Vec<libc::c_int> = vec![0, 2];
@@ -712,7 +869,7 @@ mod tests {
 
         assert_eq!(name, "kern.osrevision");
 
-        let ctl = crate::Ctl { oid };
+        let ctl = crate::Ctl::Oid(oid);
         let name = ctl
             .name()
             .expect("Could not get name of kern.osrevision sysctl.");
@@ -748,6 +905,13 @@ mod tests {
             .expect("Could notget kern.osrevision value type");
         assert_eq!(value_type, crate::CtlType::Int);
     }
+
+    /// The name must be respresentable as a C String
+    #[test]
+    fn name2oid_invalid_name() {
+        let r = super::name2oid("kern.\0.invalid.utf-8");
+        assert!(matches!(r, Err(super::SysctlError::NotFound(_))));
+    }
 }
 
 #[cfg(all(test, target_os = "freebsd"))]
@@ -762,7 +926,7 @@ mod tests_freebsd {
     }
 }
 
-#[cfg(all(test, target_os = "macos"))]
+#[cfg(all(test, any(target_os = "macos", target_os = "ios", target_os = "tvos", target_os = "visionos")))]
 mod tests_macos {
     #[test]
     fn ctl_mib() {
